@@ -35,15 +35,17 @@ do {
 };
 
 use OP::Class qw| true false |;
+use OP::Num;
 use Perl6::Subs;
 use Scalar::Util qw| blessed |;
+use Time::Local;
 
 use base qw| Time::Piece::Nonpolluting OP::Array |;
 
-use overload (
+use overload
+  %OP::Num::overload,
   '""'  => '_sprint',
-  '<=>' => '_compare',
-);
+  '<=>' => '_compare';
 
 method assert(OP::Class $class: *@rules) {
   my %parsed = OP::Type::__parseTypeArgs(
@@ -82,6 +84,17 @@ method new(OP::Class $class: Any $time) {
   return bless $self, $class;
 };
 
+method newFrom(OP::Class $class:
+  Num $year, Num $month, Num $day, Num $hour, Num $minute, Num $sec
+) {
+  return $class->new(
+    Time::Local::timelocal(
+      $sec, $minute, $hour,
+      $day, $month - 1, $year - 1900
+    )
+  );
+};
+
 #
 # Allow comparison of DateTime, Time::Piece, overloaded scalar,
 # and raw number values.
@@ -130,7 +143,7 @@ __END__
 
 =head1 NAME
 
-OP::DateTime
+OP::DateTime - Overloaded Time object class
 
 =head1 VERSION
 
@@ -140,8 +153,14 @@ OP::DateTime
 
   use OP::DateTime;
 
+From Epoch:
+
   my $time = OP::DateTime->new( time() );
 
+From YYYY MM DD hh mm ss:
+
+  my $time = OP::DateTime->newFrom(1999,12,31,23,59,59);
+
 =head1 DESCRIPTION
 
 Time object.
