@@ -58,6 +58,8 @@ our %overload = (
   '**'  => sub { "$_[0]" ** "$_[1]" },
   '=='  => sub { "$_[0]" == "$_[1]" },
   'eq'  => sub { "$_[0]" eq "$_[1]" },
+  '!='  => sub { "$_[0]" != "$_[1]" },
+  'ne'  => sub { "$_[0]" ne "$_[1]" },
 );
 
 use overload fallback => true, %overload;
